@@ -6,7 +6,8 @@
 
 #define INTEGRITY_CHECKS 1
 /* #define DEBUG_ALLOCS 1 */
-/* #define PGIS_DEBUG 1 */
+/*#define PGIS_DEBUG 1
+#define PGIS_DEBUG_CALLS 1*/
 /* #define DEBUG_CALLS 1 */
 
 /*
@@ -250,7 +251,7 @@ typedef struct
 /* MULTIPOINTTYPE */
 typedef struct
 {
-	uchar type; 
+	uchar type;  
 	BOX2DFLOAT4 *bbox;
    	uint32 SRID;	
 	int  ngeoms;
@@ -521,6 +522,7 @@ char is_worth_caching_lwgeom_bbox(const LWGEOM *);
 extern size_t lwgeom_size(const uchar *serialized_form);
 extern size_t lwgeom_size_subgeom(const uchar *serialized_form, int geom_number);
 extern size_t lwgeom_size_line(const uchar *serialized_line);
+extern size_t lwgeom_size_curve(const uchar *serialized_curve);
 extern size_t lwgeom_size_point(const uchar *serialized_point);
 extern size_t lwgeom_size_poly(const uchar *serialized_line);
 
@@ -742,6 +744,8 @@ extern LWLINE *lwgeom_getline_inspected(LWGEOM_INSPECTED *inspected, int geom_nu
 extern LWPOLY *lwgeom_getpoly(uchar *serialized_form, int geom_number);
 extern LWPOLY *lwgeom_getpoly_inspected(LWGEOM_INSPECTED *inspected, int geom_number);
 
+extern LWGEOM *lwgeom_getgeom_inspected(LWGEOM_INSPECTED *inspected, int geom_number);
+
 /*
  * this gets the serialized form of a sub-geometry
  * 1st geometry has geom_number = 0
@@ -1073,4 +1077,116 @@ extern char getMachineEndian(void);
 
 void errorIfSRIDMismatch(int srid1, int srid2);
 
+
+/* CURVETYPE */
+typedef struct
+{
+        uchar type; /* CURVETYPE */
+        BOX2DFLOAT4 *bbox;
+        uint32 SRID;
+        POINTARRAY *points; /* array of POINT(3D/3DM) */
+} LWCURVE; /* "light-weight arcline" */
+
+/* COMPOUNDTYPE */
+typedef struct
+{
+        uchar type; /* COMPOUNDTYPE */
+        BOX2DFLOAT4 *bbox;
+        uint32 SRID;
+        int ngeoms;
+        LWGEOM **geoms;
+} LWCOMPOUND; /* "light-weight compound line" */
+
+/* CURVEPOLYTYPE */
+typedef struct
+{
+        uchar type; /* CURVEPOLYTYPE */
+        BOX2DFLOAT4 *bbox;
+        uint32 SRID;
+        int nrings;
+        LWGEOM **rings; /* list of rings (list of points) */
+} LWCURVEPOLY; /* "light-weight polygon" */
+
+/* MULTICURVE */
+typedef struct
+{
+        uchar type;
+        BOX2DFLOAT4 *bbox;
+        uint32 SRID;
+        int ngeoms;
+        LWGEOM **geoms;
+} LWMCURVE;
+
+/* MULTISURFACETYPE */
+typedef struct
+{
+        uchar type;
+        BOX2DFLOAT4 *bbox;
+        uint32 SRID;
+        int ngeoms;
+        LWGEOM **geoms;
+} LWMSURFACE;
+
+#define CURVETYPE       8
+#define COMPOUNDTYPE    9
+#define CURVEPOLYTYPE   13
+#define MULTICURVETYPE          14
+#define MULTISURFACETYPE        15
+
+/******************************************************************
+ * LWCURVE functions
+ ******************************************************************/
+
+/*
+ * given the LWGEOM serialized form (or a pointer into a muli* one)
+ * construct a proper LWCURVE.
+ * serialized_form should point to the 8bit type format (with type = 2)
+ * See SERIALIZED_FORM doc
+ */
+extern LWCURVE *lwcurve_deserialize(uchar *serialized_form);
+
+/* find the size this curve would get when serialized */
+extern size_t lwcurve_serialize_size(LWCURVE *curve);
+
+/*
+ * convert this curve into its serialize form
+ * result's first char will be the 8bit type.  See serialized form doc
+ * copies data.
+ */
+extern uchar *lwcurve_serialize(LWCURVE *curve);
+
+/* same as above, writes to buf */
+extern void lwcurve_serialize_buf(LWCURVE *curve, uchar *buf, size_t *size);
+
+/*
+ * find bounding box (standard one)  zmin=zmax=0 if 2d (might change to NaN)
+ */
+extern BOX3D *lwcurve_compute_box3d(LWCURVE *curve);
+
+LWGEOM *lwcurve_add(const LWCURVE *to, uint32 where, const LWGEOM *what);
+extern int lwcurve_compute_box2d_p(LWCURVE *curve, BOX2DFLOAT4 *box);
+extern BOX3D *lwcurve_compute_box3d(LWCURVE *curve);
+extern void pfree_curve(LWCURVE  *curve);
+
+/******************************************************************
+ * LWMULTIx and LWCOLLECTION functions
+ ******************************************************************/
+
+LWCOMPOUND *lwcompound_deserialize(uchar *serialized_form);
+
+LWGEOM *lwcompound_add(const LWCOMPOUND *to, uint32 where, const LWGEOM *what);
+
+LWCURVEPOLY *lwcurvepoly_deserialize(uchar *serialized_form);
+
+LWGEOM *lwcurvepoly_add(const LWCURVEPOLY *to, uint32 where, const LWGEOM *what);
+
+LWMCURVE *lwmcurve_deserialize(uchar *serialized_form);
+
+LWGEOM *lwmcurve_add(const LWMCURVE *to, uint32 where, const LWGEOM *what);
+
+LWMSURFACE *lwmsurface_deserialize(uchar *serialized_form);
+
+LWGEOM *lwmsurface_add(const LWMSURFACE *to, uint32 where, const LWGEOM *what);
 #endif /* !defined _LIBLWGEOM_H  */
+
+double lwcircle_center(POINT4D *p1, POINT4D *p2, POINT4D *p3, POINT4D **result);
